/*
 *  linux/arch/arm/mach-ep93xx/arch.c
 *
 *  Copyright (C) 2000 Deep Blue Solutions Ltd
 * (c) Copyright 2002-2003 Cirrus Logic, Inc., Austin, Tx 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/config.h>
#include <linux/device.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/utsname.h>
#include <linux/mtd/physmap.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/memory.h>
#include <asm/arch/platform.h>
#include <asm/mach/arch.h>
#include <asm/mach/flash.h>
#include <asm/arch/ssp.h>
#include <asm/arch/crunch.h>

extern void crunch_init(void);
int crunch_is_enabled=0;

extern void ep93xx_map_io(void);
extern void ep93xx_init_irq(void);
extern struct sys_timer ep93xx_timer;

static struct flash_platform_data ep93xx_flash_data = {
	.map_name	= "cfi_probe",
	.width		= CONFIG_EP93XX_FLASH_WIDTH,
};

static struct resource ep93xx_flash_resource = {
	.start		= CONFIG_EP93XX_FLASH_BASE,
	.end		= CONFIG_EP93XX_FLASH_BASE + CONFIG_EP93XX_FLASH_SIZE - 1,
	.flags		= IORESOURCE_MEM,
};

static struct platform_device ep93xx_flash_device = {
#if defined(CONFIG_ARCH_ARMADILLO9)
	.name		= "armadillo9-nor",
#elif defined(CONFIG_ARCH_ARMADILLO210)
	.name		= "armadillo210-nor",
#elif defined(CONFIG_ARCH_ARMADILLO220) || \
      defined(CONFIG_ARCH_ARMADILLO230) || \
      defined(CONFIG_ARCH_ARMADILLO240)
	.name		= "armadillo2x0-nor",
#else
	.name		= "edb93xxflash",
#endif
	.id		= 0,
	.dev		= {
		.platform_data	= &ep93xx_flash_data,
	},
	.num_resources	= 1,
	.resource	= &ep93xx_flash_resource,
};

static struct resource ep93xx_ethernet_resources[] = {
	[0] = {
		.start		= HwRegToPhys(MAC_BASE),
		.end		= HwRegToPhys(MAC_BASE) + 0xffff,
		.flags		= IORESOURCE_MEM,
	},
	[1] = {
		.start		= IRQ_MAC,
		.end		= IRQ_MAC,
		.flags		= IORESOURCE_IRQ,
	},
};

static struct platform_device ep93xx_ethernet_device = {
	.name		= "ep93xxethernet",
	.id		= 0,
	.num_resources	= 2,
	.resource	= ep93xx_ethernet_resources,
};

#if !defined(CONFIG_ARCH_ARMADILLO210)
#if !(defined(CONFIG_ARCH_ARMADILLO220) || defined(CONFIG_ARCH_ARMADILLO230))

static struct resource ep93xx_raster_resources[] = {
	[0] = {
		.start		= HwRegToPhys(RASTER_BASE),
#ifdef CONFIG_EP93XX_GRAPHICS
		.end		= HwRegToPhys(RASTER_BASE) + 0x1ffff,
#else
		.end		= HwRegToPhys(RASTER_BASE) + 0xffff,
#endif
		.flags		= IORESOURCE_MEM,
	},
#ifdef CONFIG_EP93XX_GRAPHICS
	[1] = {
		.start		= IRQ_GRAPHICS,
		.end		= IRQ_GRAPHICS,
		.flags		= IORESOURCE_IRQ,
	},
#endif
};

static struct platform_device ep93xx_raster_device = {
	.name		= "ep93xxfb",
	.id		= 0,
#ifdef CONFIG_EP93XX_GRAPHICS
	.num_resources	= 2,
#else
	.num_resources	= 1,
#endif
	.resource	= ep93xx_raster_resources,
};
#endif

static struct resource ep93xx_usb_resources[] = {
	[0] = {
		.start		= HwRegToPhys(USB_BASE),
		.end		= HwRegToPhys(USB_BASE) + 0x8F,
		.flags		= IORESOURCE_MEM,
	},
	[1] = {
		.start		= IRQ_USH,
		.end		= IRQ_USH,
		.flags		= IORESOURCE_IRQ,
	},
};

static u64 ep93xx_usbhost_dma_mask = 0xffffffffUL;

static struct platform_device ep93xx_usb_device = {
	.name		= "ep93xxusb",
	.id		= 0,
	.num_resources	= 2,
	.resource	= ep93xx_usb_resources,
	.dev = {
		.dma_mask               = &ep93xx_usbhost_dma_mask,
		.coherent_dma_mask      = 0xffffffffUL,
	},
};

#if !(defined(CONFIG_ARCH_ARMADILLO220) || defined(CONFIG_ARCH_ARMADILLO230))
static struct resource ep93xx_i2s_resources[] = {
	[0] = {
		.start		= HwRegToPhys(I2S_BASE),
		.end		= HwRegToPhys(I2S_BASE), + 0x6F,
		.flags		= IORESOURCE_MEM,
	},
	[1] = {
		.start		= NO_IRQ,
		.end		= NO_IRQ,
		.flags		= IORESOURCE_IRQ,
	},
};

static u64 ep93xx_i2s_dma_mask = 0xffffffffUL;

static struct platform_device ep93xx_i2s_device = {
	.name		= "ep93xxi2s",
	.id		= 0,
	.num_resources	= 2,
	.resource	= ep93xx_i2s_resources,
	.dev = {
		.dma_mask               = &ep93xx_i2s_dma_mask,
		.coherent_dma_mask      = 0xffffffffUL,
	},
};

#if !defined(CONFIG_ARCG_ARMADILLO9)

static struct resource ep93xx_ac97_resources[] = {
	[0] = {
		.start		= HwRegToPhys(AC97_BASE),
		.end		= HwRegToPhys(AC97_BASE), + 0x6F,
		.flags		= IORESOURCE_MEM,
	},
	[1] = {
		.start		= NO_IRQ,
		.end		= NO_IRQ,
		.flags		= IORESOURCE_IRQ,
	},
};

static u64 ep93xx_ac97_dma_mask = 0xffffffffUL;

static struct platform_device ep93xx_ac97_device = {
	.name		= "ep93xx-ac97",
	.id		= 0,
	.num_resources	= 2,
	.resource	= ep93xx_ac97_resources,
	.dev = {
		.dma_mask               = &ep93xx_ac97_dma_mask,
		.coherent_dma_mask      = 0xffffffffUL,
	},
};

static struct resource ep93xx_gadget_resources[] = {
	[0] = {
		.start		= 0x70000000,
		.end		= 0x70000000 + 0xff,
		.flags		= IORESOURCE_MEM,
	},
#ifdef CONFIG_ARCH_EP9315A
	[1] = {
		.start		= IRQ_EXT0,
		.end		= IRQ_EXT0,
		.flags		= IORESOURCE_IRQ,
	},
#else
        [1] = {
                .start          = IRQ_EXT1,
                .end            = IRQ_EXT1,
                .flags          = IORESOURCE_IRQ,
        },

#endif

};

static u64 ep93xx_gadget_dma_mask = 0xffffffffUL;

static struct platform_device ep93xx_gadget_device = {
	.name		= "ep93xx-udc",
	.id		= 0,
	.num_resources	= 2,
	.resource	= ep93xx_gadget_resources,
	.dev = {
		.dma_mask               = &ep93xx_gadget_dma_mask,
		.coherent_dma_mask      = 0xffffffffUL,
	},
};
#endif
#endif //CONFIG_ARCH_ARMADILLO9
#endif //CONFIG_ARCH_ARMADILLO210/220/230

static struct platform_device *ep93xx_devices[] __initdata = {
	&ep93xx_flash_device,
	&ep93xx_ethernet_device,
#if !defined(CONFIG_ARCH_ARMADILLO210)
#if !(defined(CONFIG_ARCH_ARMADILLO220) || defined(CONFIG_ARCH_ARMADILLO230))
	&ep93xx_raster_device,
#endif
	&ep93xx_usb_device,
#if !(defined(CONFIG_ARCH_ARMADILLO220) || defined(CONFIG_ARCH_ARMADILLO230))
	&ep93xx_i2s_device,
#if !defined(CONFIG_ARCH_ARMADILLO9)
	&ep93xx_ac97_device,
	&ep93xx_gadget_device,
#endif
#endif //CONFIG_ARCH_ARMADILLO9
#endif //CONFIG_ARCH_ARMADILLO210
};

static void __init
ep93xx_init(void)
{
	unsigned int uiTemp;
#if !(defined(CONFIG_ARCH_ARMADILLO9) || defined(CONFIG_ARCH_ARMADILLO2X0))
	int SSP_Handle;
#endif

	/*
	 * Add the platform devices.
	 */
	platform_add_devices(ep93xx_devices, ARRAY_SIZE(ep93xx_devices));

	/*
	 * Enable halt mode.
	 */
	uiTemp = inl(SYSCON_DEVCFG) | SYSCON_DEVCFG_SHena;
	SysconSetLocked(SYSCON_DEVCFG, uiTemp);

	/*
	 * Set the bus priority.
	 */
	outl(BMAR_PRIORD_02, SYSCON_BMAR);

#if !(defined(CONFIG_ARCH_ARMADILLO9) || defined(CONFIG_ARCH_ARMADILLO2X0))
	/*
	 * Get the hostname from the SPI FLASH if it has been programmed.
	 */
	SSP_Handle = SSPDriver->Open(SERIAL_FLASH, 0);
	if(SSP_Handle != -1) {
		SSPDriver->Read( SSP_Handle, 0x1000, &uiTemp );
		if (uiTemp == 0x43414d45) {
			SSPDriver->Read( SSP_Handle, 0x1010, &uiTemp );
			system_utsname.nodename[0] = uiTemp & 255;
			system_utsname.nodename[1] = (uiTemp >> 8) & 255;
			system_utsname.nodename[2] = (uiTemp >> 16) & 255;
			system_utsname.nodename[3] = uiTemp >> 24;
			SSPDriver->Read( SSP_Handle, 0x1014, &uiTemp );
			system_utsname.nodename[4] = uiTemp & 255;
			system_utsname.nodename[5] = (uiTemp >> 8) & 255;
			system_utsname.nodename[6] = (uiTemp >> 16) & 255;
			system_utsname.nodename[7] = uiTemp >> 24;
			SSPDriver->Read( SSP_Handle, 0x1018, &uiTemp );
			system_utsname.nodename[8] = uiTemp & 255;
			system_utsname.nodename[9] = (uiTemp >> 8) & 255;
			system_utsname.nodename[10] = (uiTemp >> 16) & 255;
			system_utsname.nodename[11] = uiTemp >> 24;
			SSPDriver->Read( SSP_Handle, 0x101c, &uiTemp );
			system_utsname.nodename[12] = uiTemp & 255;
			system_utsname.nodename[13] = (uiTemp >> 8) & 255;
			system_utsname.nodename[14] = (uiTemp >> 16) & 255;
			system_utsname.nodename[15] = uiTemp >> 24;
			system_utsname.nodename[16] = 0;
		}
		SSPDriver->Close( SSP_Handle );
	}
#endif

#ifdef CONFIG_EP93XX_FPU
    crunch_enable();
    crunch_init();
    crunch_is_enabled=1;
#endif

}

/* Machine Descriptor created by macros in asm-arm/mach/arch.h
    bootmem: phys ram base, phys io base, virt io base
    bootpara: phys param base  */

#ifdef CONFIG_ARCH_EDB9301
MACHINE_START(EDB9301, "edb9301")
    MAINTAINER("Cirrus Logic")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#ifdef CONFIG_ARCH_EDB9302
MACHINE_START(EDB9302, "edb9302")
    MAINTAINER("Cirrus Logic")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#ifdef CONFIG_ARCH_EDB9307
MACHINE_START(EDB9307, "edb9307")
    MAINTAINER("Cirrus Logic")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#ifdef CONFIG_ARCH_EDB9312
MACHINE_START(EDB9312, "edb9312")
    MAINTAINER("Cirrus Logic")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#ifdef CONFIG_ARCH_EDB9315
MACHINE_START(EDB9315, "edb9315")
    MAINTAINER("Cirrus Logic")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#ifdef CONFIG_ARCH_EDB9315A
MACHINE_START(EDB9315A, "edb9315A")
    MAINTAINER("Cirrus Logic")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#if defined(CONFIG_ARCH_ARMADILLO9)

#ifdef CONFIG_ARMADILLO9_ISA
volatile u8 miscreg_intmr1  = 0x0;
volatile u8 miscreg_intmr2  = 0x0;
volatile u8 miscreg_intmr3  = 0x0;
volatile u8 miscreg_isamode = 0x0;
#endif

MACHINE_START(ARMADILLO9, "Armadillo-9")
    MAINTAINER("Atmark Techno, Inc.")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#if defined(CONFIG_ARCH_ARMADILLO210)

MACHINE_START(ARMADILLO210, "Armadillo-210")
    MAINTAINER("Atmark Techno, Inc.")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#if defined(CONFIG_ARCH_ARMADILLO220)

MACHINE_START(ARMADILLO220, "Armadillo-220")
    MAINTAINER("Atmark Techno, Inc.")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#if defined(CONFIG_ARCH_ARMADILLO230)

MACHINE_START(ARMADILLO230, "Armadillo-230")
    MAINTAINER("Atmark Techno, Inc.")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif

#if defined(CONFIG_ARCH_ARMADILLO240)

MACHINE_START(ARMADILLO240, "Armadillo-240")
    MAINTAINER("Atmark Techno, Inc.")
    BOOT_MEM( CONFIG_EP93XX_PHYS_ADDR, 0x80000000, 0xff000000 )
    BOOT_PARAMS( CONFIG_EP93XX_PARAMS_PHYS )
    MAPIO(ep93xx_map_io)
    INITIRQ(ep93xx_init_irq)
    .timer = &ep93xx_timer,
    INIT_MACHINE(ep93xx_init)
MACHINE_END
#endif
